#include <iostream>
#include <boost/filesystem.hpp>
#include <boost/filesystem/fstream.hpp>
#include <clx/hexdump.h>
#include <clx/ini.h>
#include <cliff/font/true_type.h>
#include <cliff/font/true_type_collection.h>

#define FONT_DIR "C:\\Windows\\Fonts"

template <class MapT>
void put_element(const std::string& path, MapT& v, clx::ini& dest, size_t index = 0) {
    if (dest.find("General") == dest.end()) dest.insert("General");
    for (typename MapT::iterator pos = v.begin(); pos != v.end(); ++pos) {
        if (pos->second.fullname().empty()) continue;
        
        if (pos->first == 0) {
            std::stringstream ss;
            ss << path << ":" << index;
            dest["General"][pos->second.fullname()] = ss.str();
        }
        else if (pos->first == 1041) { // {̃tHg
            if (dest.find("Japanese") == dest.end()) dest.insert("Japanese");
            dest["Japanese"][pos->second.fullname()] = v[0].fullname();
        }
    }
}

namespace fs = boost::filesystem;

int main(int argc, char* argv[]) {
    clx::ini dest;
    
    fs::path sdir = (argc > 1) ? std::string(argv[1]) : std::string(FONT_DIR);
    fs::directory_iterator pos(sdir);
    fs::directory_iterator last;
    for(; pos != last; ++pos) {
        fs::path pabs = fs::complete(pos->path());
        fs::ifstream ifs(pabs, std::ios::binary);
        std::string ext = fs::extension(pabs);
        clx::downcase(ext);
        if (ext == ".ttf") {
            cliff::true_type f;
            if (f.read(ifs)) put_element(pabs.string(), f.names(), dest);
        }
        else if (ext == ".ttc") {
            cliff::true_type_collection f;
            if (f.read(ifs)) {
                for (size_t i = 0; i < f.size(); ++i) {
                    put_element(pabs.string(), f.at(i).names(), dest, i);
                }
            }
        }
    }
    
    dest.write("fontmap.ini");
    
    return 0;
}
