/* ------------------------------------------------------------------------- */
/*
 *  true_type_collection.h
 *
 *  Copyright (c) 2009 - 2010, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Copyright (c) 2000,2001,2002,2003,2004,2005,2006,2007,2008,2009
 *  by George Williams 
 *
 *  The source code is derived from the FontForge Project. (See the
 *  following URL: http://fontforge.sourceforge.net/)
 *
 *  The source code uses the TrickLibrary. (See the following URL:
 *  http://tricklib.com/cxx/ex/babel/)
 *
 *  Last-modified: Thu 07 Jan 2010 16:49:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLIFF_FONT_TRUE_TYPE_COLLECTION_H
#define CLIFF_FONT_TRUE_TYPE_COLLECTION_H

#include <cliff/utility.h>
#include <cliff/font/true_type.h>

namespace cliff {
	/* --------------------------------------------------------------------- */
	/*
	 *  true_type
	 *
	 *  TrueTypeCollection (*.ttc) 形式のファイルを解析するクラス．
	 *  *.ttf には，複数の TrueType (*.ttc) の情報が格納されている．
	 */
	/* --------------------------------------------------------------------- */
	class true_type_collection : public clx::vector_wrapper<true_type> {
	private:
		typedef clx::vector_wrapper<true_type> super;
	public:
		true_type_collection() : super() {}
		
		explicit true_type_collection(istream_type& in) :
			super() {
			this->read(in);
		}
		
		explicit true_type_collection(const string_type& path) :
			super() {
			this->read(path);
		}
		
		virtual ~true_type_collection() throw() {}
		
		bool read(istream_type& in) {
			size_type sig = 0;
			if (!clx::get(in, sig, clx::endian::big) ||
				sig != clx::combine4_<'t', 't', 'c', 'f'>::value) {
				return false;
			}
			in.seekg(4, std::ios_base::cur); // reserve?
			
			size_type count = 0;
			if (!clx::get(in, count, clx::endian::big)) return false;
			
			std::vector<size_type> offset;
			for (size_type i = 0; i < count; ++i) {
				size_type x = 0;
				if (!clx::get(in, x, clx::endian::big)) return false;
				offset.push_back(x);
			}
			
			for (size_type i = 0; i < offset.size(); ++i) {
				true_type elem;
				in.seekg(offset.at(i));
				if (!elem.read(in)) return false;
				member.push_back(elem);
			}
			
			return true;
		}
		
		bool read(const string_type& path) {
			std::basic_ifstream<char_type> ifs(path.c_str());
			if (ifs.fail()) return false;
			return this->read(ifs);
		}
	};
}

#endif // CLIFF_FONT_TRUE_TYPE_COLLECTION_H
