/* ------------------------------------------------------------------------- */
/*
 *  utility.h
 *
 *  Copyright (c) 2009, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Wed 21 Oct 2009 14:01:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLIFF_UTILITY_H
#define CLIFF_UTILITY_H

#include <string>
#include <ostream>
#include <clx/shared_ptr.h>
#include <clx/format.h>
#include <clx/unit.h>

namespace cliff {
	using clx::shared_ptr;
	
	/* --------------------------------------------------------------------- */
	/*
	 *  cliff での基本型の定義． cliff では，基本的に文字列は UTF-8 の
	 *  char で扱い， wchar_t は使わないように設計している．
	 */
	/* --------------------------------------------------------------------- */
	typedef char char_type;
	typedef unsigned char byte_type;
	typedef unsigned short word_type;
	typedef std::size_t size_type;
	typedef std::basic_string<char_type> string_type;
	typedef std::basic_istream<char_type> istream_type;
	typedef std::basic_ostream<char_type> ostream_type;
	
	/* --------------------------------------------------------------------- */
	//  format を使用する頻度が高いので，format だけは共通して定義しておく．
	/* --------------------------------------------------------------------- */
	typedef clx::basic_format<char_type> format;
	
	/* --------------------------------------------------------------------- */
	/*
	 *  page_size
	 *
	 *  ページの幅と高さを定義した構造体．単位はポイント (pt)．
	 */
	/* --------------------------------------------------------------------- */
	template <unsigned int W, unsigned int H>
	struct page_size {
		typedef unsigned int uint_t;
		
		static const uint_t width = W;
		static const uint_t height = H;
	};
	
	typedef page_size< 842, 1191> a3;
	typedef page_size< 595,  842> a4;
	typedef page_size< 420,  595> a5;
	
	typedef page_size< 729, 1032> b4;
	typedef page_size< 516,  729> b5;
	
	typedef page_size< 612,  792> letter;
	typedef page_size< 612, 1008> legal;
	typedef page_size<1223,  792> ledger;
	typedef page_size< 792, 1224> tabloid;
	
	typedef page_size< 720,  540> slide; // MS PowerPoint のデフォルト
	
	/* --------------------------------------------------------------------- */
	//  他の各単位からポイント (pt) に変換するためのクラス群．
	/* --------------------------------------------------------------------- */
	typedef clx::unit<72, 1>    inch;
	typedef clx::unit<1, 20>    twip; // MS Word, MS Excel で使用される単位（2003 以前）
	typedef clx::unit<1, 12700> emu;  // MS Office 全般で使用される単位
}

#endif // CLIFF_UTILITY_H
